 //////////////////////////////////////////////
//
// SourceMod Script
//
// DoD CloseCombat Source
//
// Developed by FeuerSturm
//
//////////////////////////////////////////////
//
//
// USAGE:
// ======
//
//
// CVARs:
// ------
//
// dod_closecombat_source <1/2/3/0>			=	set CloseCombat Weapons
//												1 = Melee only (Knife/Spade)
//												2 = Pistols only (Colt/P38)
//												3 = Melee & Pistols
//												0 = disabled
//
// dod_closecombat_sounds <1/0>				=	enable/disable using sounds for headshots,
//												melee kills and announcement
//
// dod_closecombat_announce <1/0>			=	enable/disable displaying a hint message about
//												the CloseCombat setup for spawning players
//
// dod_closecombat_pistolclips <#>			=	set amount of clips to hand out for the pistol
//
// dod_closecombat_meleebonushp <#/0>		=	set amount of health points to add for melee kills
//												0 = disable
//
// dod_closecombat_headshotbonushp <#/0>	=	set amount of health points to add for pistol headshots
//												0 = disable
//
// dod_closecombat_glowplayers <1/0>		=	enable/disable making players glow in their team's color
//
//
// dod_closecombat_fraggrenades <#/0>		=	set number of frag grenades to give to each player
//												0 to disable fraggrenades!
//
// dod_closecombat_smokegrenades <#/0>		=	set number of smoke grenades to give to each player
//												0 to disable smokegrenades!
//
//
//
// CHANGELOG:
// ==========
// 
// - 12 October 2008 - Version 1.0
//   Initial Release
//
// - 16 November 2008 - Version 1.1
//   New Features:
//   * smoke grenades and/or frag grenades can
//     be added to players closecombat equipment
//     as well!
//     (see new cvars "dod_closecombat_fraggrenades"
//      and "dod_closecombat_smokegrenades")
//   * added Multi-Language support!
//   Bugfixes:
//   * fixed players still glowing after disabling
//     the plugin
//   * fixed "Client X is not in game" errors
//
//
#include <sourcemod>
#include <sdktools>

#define PLUGIN_VERSION "1.1"

#define SPEC 1
#define ALLIES 2
#define AXIS 3

public Plugin:myinfo = 
{
	name = "DoD CloseCombat Source", 
	author = "FeuerSturm", 
	description = "Sidearms and/or Melee Weapons only (+ grenades)!!", 
	version = PLUGIN_VERSION, 
	url = "http://www.dodsourceplugins.net"
}

new String:CloseCombatSound[4][] =  { "", "", "player/american/us_dropweapons.wav", "player/german/ger_dropweapons.wav" }
new String:WpnMelee[4][] =  { "", "", "weapon_amerknife", "weapon_spade" }
new String:WpnPistol[4][] =  { "", "", "weapon_colt", "weapon_p38" }
new String:WpnFragGrenade[4][] =  { "", "", "weapon_frag_us", "weapon_frag_ger" }
new String:WpnSmokeGrenade[4][] =  { "", "", "weapon_smoke_us", "weapon_smoke_ger" }
new PistolOffset[4] =  { 0, 0, 4, 8 }
new PistolAmmo[4] =  { 0, 0, 7, 8 }
new FragGrenadeOffset[4] =  { 0, 0, 52, 56 }
new SmokeGrenadeOffset[4] =  { 0, 0, 68, 72 }
new String:Headshot[3][] =  { "player/headshot1.wav", "player/headshot2.wav", "player/headshot3.wav" }
new String:Melee[] =  { "misc/freeze_cam.wav" }
new String:Status[4][] =  { "DISABLED", "Melee Weapons ONLY!", "Pistols ONLY!", "CloseCombat Weapons ONLY!" }

new g_nomessage[MAXPLAYERS + 1]
new bool:IsGlowing[MAXPLAYERS + 1]

new Handle:CCStatus = INVALID_HANDLE
new Handle:CCSounds = INVALID_HANDLE
new Handle:CCAnnounce = INVALID_HANDLE
new Handle:CCFragGrenades = INVALID_HANDLE
new Handle:CCSmokeGrenades = INVALID_HANDLE
new Handle:CCPistolClips = INVALID_HANDLE
new Handle:CCMeleeBonus = INVALID_HANDLE
new Handle:CCHeadshotBonus = INVALID_HANDLE
new Handle:CCGlowPlayers = INVALID_HANDLE

public OnPluginStart()
{
	CreateConVar("dod_closecombat_version", PLUGIN_VERSION, "DoD CloseCombat Source Version (DO NOT CHANGE!)", FCVAR_DONTRECORD | FCVAR_PLUGIN | FCVAR_SPONLY | FCVAR_REPLICATED | FCVAR_NOTIFY)
	SetConVarString(FindConVar("dod_closecombat_version"), PLUGIN_VERSION)
	CCStatus = CreateConVar("dod_closecombat_source", "0", "<1/2/3/0> = set CloseCombat Weapons  -  1=Melee only  -  2=Pistols only  -  3=Melee & Pistols  -  0=disabled!", FCVAR_PLUGIN)
	CCSounds = CreateConVar("dod_closecombat_sounds", "1", "<1/0> = enable/disable using sounds for headshots, melee kills and announcement", FCVAR_PLUGIN)
	CCAnnounce = CreateConVar("dod_closecombat_announce", "1", "<1/0> = enable/disable displaying a hint message about the CloseCombat setup for spawning players", FCVAR_PLUGIN)
	CCPistolClips = CreateConVar("dod_closecombat_pistolclips", "50", "<#> = set amount of clips to hand out for the pistol", FCVAR_PLUGIN)
	CCMeleeBonus = CreateConVar("dod_closecombat_meleebonushp", "25", "<#/0> = set amount of health points to add for melee kills  -  0=disable", FCVAR_PLUGIN)
	CCHeadshotBonus = CreateConVar("dod_closecombat_headshotbonushp", "10", "<#/0> = set amount of health points to add for pistol headshots  -  0=disable", FCVAR_PLUGIN)
	CCGlowPlayers = CreateConVar("dod_closecombat_glowplayers", "1", "<1/0> = enable/disable making players glow in their team's color", FCVAR_PLUGIN)
	CCFragGrenades = CreateConVar("dod_closecombat_fraggrenades", "3", "<0/#> = set number of frag grenades to give to each player  -  0 to disable fraggrenades!", FCVAR_PLUGIN)
	CCSmokeGrenades = CreateConVar("dod_closecombat_smokegrenades", "2", "<0/#> = set number of smoke grenades to give to each player  -  0 to disable smokegrenades!", FCVAR_PLUGIN)
	HookEvent("player_team", Surpress_TeamMSG, EventHookMode_Pre)
	HookEvent("dod_stats_weapon_attack", EventWeaponAttack, EventHookMode_Pre)
	HookEventEx("dod_stats_player_damage", EventPlayerDamage, EventHookMode_Post)
	HookEventEx("player_spawn", EventPlayerSpawn, EventHookMode_Post)
	PrecacheSound(Headshot[0])
	PrecacheSound(Headshot[1])
	PrecacheSound(Headshot[2])
	PrecacheSound(Melee)
	HookConVarChange(CCStatus, CCStatusChange)
	AutoExecConfig(true, "dod_closecombat_source", "dod_closecombat_source")
	LoadTranslations("dod_closecombat_source.txt")
}

public OnClientPostAdminCheck(client)
{
	g_nomessage[client] = 0
	IsGlowing[client] = false
}

public OnClientDisconnect(client)
{
	g_nomessage[client] = 0
	IsGlowing[client] = false
}

public Action:Surpress_TeamMSG(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"))
	if (g_nomessage[client] == 1)
	{
		g_nomessage[client] = 0
		return Plugin_Handled
	}
	return Plugin_Continue
}

public CCStatusChange(Handle:convar, const String:oldValue[], const String:newValue[])
{
	new oldvalue = StringToInt(oldValue)
	new newvalue = StringToInt(newValue)
	if (oldvalue != newvalue)
	{
		for (new client = 1; client < GetMaxClients(); client++)
		{
			if (IsClientInGame(client) && IsPlayerAlive(client))
			{
				new team = GetClientTeam(client)
				g_nomessage[client] = 1
				ChangeClientTeam(client, SPEC)
				SecretTeamSwitch(client, team)
			}
		}
		PrintToChatAll("\x04[DoD CloseCombat] \x01%T", "StatusChange", LANG_SERVER, Status[newvalue])
	}
}

stock SecretTeamSwitch(client, team)
{
	g_nomessage[client] = 1
	ChangeClientTeam(client, team)
	ShowVGUIPanel(client, team == AXIS ? "class_ger" : "class_us", INVALID_HANDLE, false)
}

public Action:EventPlayerDamage(Handle:event, const String:name[], bool:dontBroadcast)
{
	new victim = GetClientOfUserId(GetEventInt(event, "victim"))
	new attacker = GetClientOfUserId(GetEventInt(event, "attacker"))
	new weapon = GetEventInt(event, "weapon")
	new Hitgroup = GetEventInt(event, "hitgroup")
	new attackerteam = GetClientTeam(attacker)
	new victimteam = GetClientTeam(victim)
	new status = GetConVarInt(CCStatus)
	if (status == 0 || GetClientHealth(victim) > 0 || attacker == victim || attackerteam == victimteam || attacker == 0 || victim == 0)
	{
		return Plugin_Continue
	}
	new sounds = GetConVarInt(CCSounds)
	if ((weapon == 1 || weapon == 2) && (status == 1 || status == 3))
	{
		new meleebonus = GetConVarInt(CCMeleeBonus)
		if (meleebonus > 0)
		{
			SetEntityHealth(attacker, GetClientHealth(attacker) + meleebonus)
			PrintToChat(attacker, "\x04[DoD CloseCombat] \x01%T", "MeleeKillBonus", attacker, meleebonus)
		}
		if (sounds == 1)
		{
			EmitSoundToClient(attacker, Melee, SOUND_FROM_PLAYER, SNDCHAN_WEAPON, SNDLEVEL_NORMAL, SND_NOFLAGS, SNDVOL_NORMAL, SNDPITCH_NORMAL)
		}
		return Plugin_Continue
	}
	if (Hitgroup == 1 && (weapon == 3 || weapon == 4) && (status == 2 || status == 3))
	{
		new headshotbonus = GetConVarInt(CCHeadshotBonus)
		if (headshotbonus > 0)
		{
			SetEntityHealth(attacker, GetClientHealth(attacker) + headshotbonus)
			PrintToChat(attacker, "\x04[DoD CloseCombat] \x01%T", "HeadshotBonus", attacker, headshotbonus)
		}
		if (sounds == 1)
		{
			EmitSoundToClient(attacker, Headshot[GetRandomInt(0, 2)], SOUND_FROM_PLAYER, SNDCHAN_WEAPON, SNDLEVEL_NORMAL, SND_NOFLAGS, SNDVOL_NORMAL, SNDPITCH_NORMAL)
		}
		return Plugin_Continue
	}
	return Plugin_Continue
}

public Action:EventWeaponAttack(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "attacker"))
	new weapon = GetEventInt(event, "weapon")
	new status = GetConVarInt(CCStatus)
	new fraggrenades = GetConVarInt(CCFragGrenades)
	new smokegrenades = GetConVarInt(CCSmokeGrenades)
	if (status == 0 || weapon < 5 || (fraggrenades != 0 && (weapon >= 19 && weapon <= 22)) || (smokegrenades != 0 && (weapon == 23 || weapon == 24)))
	{
		return Plugin_Continue
	}
	CloseCombatPlayer(client)
	if (GetConVarInt(CCAnnounce) == 1)
	{
		CreateTimer(0.1, ShowMsg, client, TIMER_FLAG_NO_MAPCHANGE)
	}
	return Plugin_Continue
}

public Action:EventPlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"))
	if (IsClientInGame(client) && IsPlayerAlive(client) && GetClientTeam(client) > 1)
	{
		if (IsGlowing[client])
		{
			SetEntityRenderColor(client)
			IsGlowing[client] = false
		}
		if (GetConVarInt(CCStatus) != 0)
		{
			CloseCombatPlayer(client)
			if (GetConVarInt(CCAnnounce) == 1)
			{
				CreateTimer(0.1, ShowMsg, client, TIMER_FLAG_NO_MAPCHANGE)
			}
			return Plugin_Continue
		}
	}
	return Plugin_Continue
}

public Action:ShowMsg(Handle:timer, any:client)
{
	if (IsClientInGame(client) && IsPlayerAlive(client) && GetClientTeam(client) > 1)
	{
		PrintHintText(client, "%s", Status[GetConVarInt(CCStatus)])
	}
	return Plugin_Handled
}

public Action:CloseCombatPlayer(client)
{
	if (IsClientInGame(client) && IsPlayerAlive(client))
	{
		new team = GetClientTeam(client)
		if (GetConVarInt(CCGlowPlayers) == 1)
		{
			if (team == ALLIES)
			{
				SetEntityRenderColor(client, 0, 255, 0, 255)
			}
			else
			{
				SetEntityRenderColor(client, 255, 0, 0, 255)
			}
			IsGlowing[client] = true
		}
		if (GetConVarInt(CCAnnounce) == 1 && GetConVarInt(CCSounds) == 1)
		{
			EmitSoundToClient(client, CloseCombatSound[team], SOUND_FROM_PLAYER, SNDCHAN_VOICE, SNDLEVEL_NORMAL, SND_NOFLAGS, SNDVOL_NORMAL, SNDPITCH_NORMAL)
		}
		StripWeapons(client)
		CreateTimer(0.1, GiveCloseCombat, client, TIMER_FLAG_NO_MAPCHANGE)
	}
	return Plugin_Handled
}

public Action:GiveCloseCombat(Handle:timer, any:client)
{
	if (IsClientInGame(client) && IsPlayerAlive(client) && GetClientTeam(client) > 1)
	{
		new ammo_offset = FindSendPropOffs("CDODPlayer", "m_iAmmo")
		new status = GetConVarInt(CCStatus)
		new fraggrenades = GetConVarInt(CCFragGrenades)
		new smokegrenades = GetConVarInt(CCSmokeGrenades)
		new team = GetClientTeam(client)
		if (fraggrenades > 0)
		{
			GivePlayerItem(client, WpnFragGrenade[team])
			SetEntData(client, ammo_offset + FragGrenadeOffset[team], fraggrenades, 4, true)
		}
		if (smokegrenades > 0)
		{
			GivePlayerItem(client, WpnSmokeGrenade[team])
			SetEntData(client, ammo_offset + SmokeGrenadeOffset[team], smokegrenades, 4, true)
		}
		if (status == 1 || status == 3)
		{
			GivePlayerItem(client, WpnMelee[team])
		}
		if (status == 2 || status == 3)
		{
			new clips = GetConVarInt(CCPistolClips) * PistolAmmo[team]
			GivePlayerItem(client, WpnPistol[team])
			SetEntData(client, ammo_offset + PistolOffset[team], clips, 4, true)
		}
	}
	return Plugin_Handled
}

public Action:StripWeapons(client)
{
	for (new i = 0; i < 4; i++)
	{
		new weapon = GetPlayerWeaponSlot(client, i)
		if (weapon != -1)
		{
			RemovePlayerItem(client, weapon)
			RemoveEdict(weapon)
		}
	}
	return Plugin_Handled
} 